/* foXcommander - foXdesktop file manager
 * Copyright (C) 2000-2001 Josua Groeger derived from XWC (C) 1999-2000 Maxim Baranov
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <fox/fx.h>
#include <fox/fxkeys.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <sys/wait.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#if defined(linux)
#include <mntent.h>
#endif

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#include "Attributes.h"
#include "Panel.h"
#include "FDAppMainWindow.h"

extern FXStringDict*      devices;
int du(char *path, off_t &size);
int Size(FXchar* size,FXchar *newsize)
{
    int rc=0;
    FXchar suf[]="bytes";
    long lsize = atol(size);
    float fsize=0.0;
    
    if(lsize>10000000)
    {
        fsize=lsize/1048576.0;
        strcpy(suf,"MBytes");
        rc=1;
    }
    else if(lsize>10000)
    {
        fsize=lsize/1024.0;
        strcpy(suf,"KBytes");
        rc=1;
    }
    if(rc)
    {
        if(fsize==(int)fsize)
            sprintf(newsize,"%.0f %s",fsize,suf);
        else
            sprintf(newsize,"%.2f %s",fsize,suf);
    }
    else
        sprintf(newsize,"%ld %s",lsize,suf);

    return rc;
}

// Map
FXDEFMAP(PermFrame) PermFrameMap[]={
/*
    FXMAPFUNC(SEL_COMMAND,S_IRUSR,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IWUSR,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IXUSR,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IRGRP,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IWGRP,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IXGRP,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IROTH,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IWOTH,::onCmdCheck),
    FXMAPFUNC(SEL_COMMAND,S_IXOTH,::onCmdCheck),
*/
};

// Object implementation
FXIMPLEMENT(PermFrame,FXVerticalFrame,PermFrameMap,ARRAYNUMBER(PermFrameMap))

PermFrame::PermFrame(FXComposite* parent,FXObject *target):
FXVerticalFrame(parent,FRAME_THICK|FRAME_RAISED)
{
    FXHorizontalFrame *accessframe=new FXHorizontalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *chmodframe=new FXHorizontalFrame(this,PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXGroupBox *group1=new FXGroupBox(accessframe,"User",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
    ur=new FXCheckButton(group1,"Read",target,S_IRUSR);
    uw=new FXCheckButton(group1,"Write",target,S_IWUSR);
    ux=new FXCheckButton(group1,"Execute",target,S_IXUSR);
    FXGroupBox *group2=new FXGroupBox(accessframe,"Group",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
    gr=new FXCheckButton(group2,"Read",target,S_IRGRP);
    gw=new FXCheckButton(group2,"Write",target,S_IWGRP);
    gx=new FXCheckButton(group2,"Execute",target,S_IXGRP);
    FXGroupBox *group3=new FXGroupBox(accessframe,"Others",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
    cb_or=new FXCheckButton(group3,"Read",target,S_IROTH);
    ow=new FXCheckButton(group3,"Write",target,S_IWOTH);
    ox=new FXCheckButton(group3,"Execute",target,S_IXOTH);
	
    FXGroupBox *group4=new FXGroupBox(chmodframe,"Command",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    set=new FXRadioButton(group4,"Set marked",target,FXAttributesBox::ID_SET);
    clear=new FXRadioButton(group4,"Clear marked",target,FXAttributesBox::ID_CLEAR);
    add=new FXRadioButton(group4,"Add marked",target,FXAttributesBox::ID_ADD);

    rec = new FXCheckButton(group4,"Recursive folders",NULL,0);
    dironly = new FXCheckButton(group4,"Folders only",NULL,0);
    dironly->disable();

    FXGroupBox *group5=new FXGroupBox(chmodframe,"Owner",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(group5,"User");
    user = new FXComboBox(group5,8,5,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);
    new FXLabel(group5,"Group");
    grp = new FXComboBox(group5,8,5,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);

    struct passwd* pwde;
    while(pwde = getpwent())
    {
        user->appendItem(pwde->pw_name);
    }
    endpwent();

    struct group* grpe;
    while(grpe = getgrent())
    {
        grp->appendItem(grpe->gr_name);
    }
    endgrent();

    
}
// Map
FXDEFMAP(FXAttributesBox) FXAttributesBoxMap[]={
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_ACCEPT,FXAttributesBox::onCmdAcceptSingle),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_ACCEPT_MULT,FXAttributesBox::onCmdAcceptMult),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_CANCEL,FXAttributesBox::onCmdCancel),
  FXMAPFUNC(SEL_UPDATE,0,FXAttributesBox::onUpdWait),

  FXMAPFUNC(SEL_COMMAND,S_IRUSR,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IWUSR,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IXUSR,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IRGRP,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IWGRP,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IXGRP,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IROTH,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IWOTH,FXAttributesBox::onCmdCheck),
  FXMAPFUNC(SEL_COMMAND,S_IXOTH,FXAttributesBox::onCmdCheck),

  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_SET,FXAttributesBox::onCmdCommand),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_CLEAR,FXAttributesBox::onCmdCommand),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_ADD,FXAttributesBox::onCmdCommand),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_BIG_ICON,FXAttributesBox::onCmdBrowseIcon),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_MINI_ICON,FXAttributesBox::onCmdBrowseIcon),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_BROWSE_OPEN,FXAttributesBox::onCmdBrowse),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_BROWSE_VIEW,FXAttributesBox::onCmdBrowse),
  FXMAPFUNC(SEL_COMMAND,FXAttributesBox::ID_BROWSE_EDIT,FXAttributesBox::onCmdBrowse),
};
// Object implementation
FXIMPLEMENT(FXAttributesBox,FXDialogBox,FXAttributesBoxMap,ARRAYNUMBER(FXAttributesBoxMap))

// Construct window for one file
FXAttributesBox::FXAttributesBox(FXApp *app,FXString file,FXString path): FXDialogBox(app,"Attributes", DECOR_TITLE|DECOR_BORDER/*,0,0,350,200*/)
{
    long filesize;
    char file_info[1024];
    char grpid[64],usrid[64],mod[32],changed[32],accessed[32];
    struct stat info;
    FXString extension;
    int islink = 0;
    char filename[1024],linkto[MAXPATHLEN+1],*e,name[1024];
//    rec = 0;
    /* Buttons */
    FXHorizontalFrame *buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,0,0,10,10,5,5);

    /* Contents */
    FXVerticalFrame *contents=new FXVerticalFrame(this,LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);

    /* Cancel */
    new FXButton(buttons,"&Cancel",NULL,this,FXAttributesBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
    /* Accept  */
    FXButton *ok = new FXButton(buttons,"&Accept",NULL,this,FXAttributesBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
    ok->addHotKey(KEY_Return);
    /* Switcher */    
    FXTabBook *tabbook = new FXTabBook(contents,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);        

    /* First item is General */
    new FXTabItem(tabbook,"&General",NULL);
    FXPacker *genpack = new FXPacker(tabbook,FRAME_THICK|FRAME_RAISED);
    FXGroupBox *generalgroup=new FXGroupBox(genpack,NULL,FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXVerticalFrame *generalframe=new FXVerticalFrame(generalgroup,LAYOUT_FILL_X|LAYOUT_FILL_Y);
    /* Second item is Access Permissions */
    new FXTabItem(tabbook,"&Permissions",NULL);
    perm = new PermFrame(tabbook,this);

    // Third tab - file associations
   FXTabItem *fassoctab = new FXTabItem(tabbook,"&File Associations",NULL);
   FXPacker *fassocpack = new FXPacker(tabbook,FRAME_THICK|FRAME_RAISED);
   FXGroupBox *fassocgroup=new FXGroupBox(fassocpack,NULL,FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   FXVerticalFrame *contassoc=new FXVerticalFrame(fassocgroup,LAYOUT_FILL_X|LAYOUT_FILL_Y);//,FRAME_THICK|FRAME_RAISED);
   FXMatrix *matrix = new FXMatrix(contassoc,3,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   fassoctab->disable();

   new FXLabel(matrix,"Extension:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   ext = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXLabel(matrix,"",NULL,0);
   
   new FXLabel(matrix,"Description:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   descr = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXLabel(matrix,"",NULL,0);

   new FXLabel(matrix,"Open:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   open = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXButton(matrix," Browse...",NULL,this,ID_BROWSE_OPEN,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,0,0);
//   new FXLabel(matrix,"",NULL,0);

   int is_ar = FALSE;
   char *viewlbl = "View:";
   char *editlbl = "Edit:";
   e = strrchr(file.text(),'.');
   if(e)
       if(!strcmp(e, ".gz") || !strcmp(e,".tgz") ||
          !strcmp(e,".tar") || !strcmp(e,".bz2") ||
          !strcmp(e,".zip") || !strcmp(e,"ZIP"))
       {
           is_ar = TRUE;// archive
           viewlbl = "Extract:";
       }
#if defined(linux)
       else if(!strcmp(e,".rpm"))
       {
           viewlbl = "Install:";
           editlbl = "Upgrade:";
       }
#endif

   new FXLabel(matrix,viewlbl,NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   view = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXButton(matrix," Browse...",NULL,this,ID_BROWSE_VIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,0,0);
//   new FXLabel(matrix,"",NULL,0);

   if(!is_ar)
   {
       new FXLabel(matrix,editlbl,NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
       edit = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
       new FXButton(matrix," Browse...",NULL,this,ID_BROWSE_EDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,0,0);
       //   new FXLabel(matrix,"",NULL,0);
   }
   else
       edit = NULL;

   new FXLabel(matrix,"Big Icon:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   bigic = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXButton(matrix," Browse...",NULL,this,ID_BIG_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,0,0);
   
   new FXLabel(matrix,"Mini Icon:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   miniic = new FXTextField(matrix,10,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
   new FXButton(matrix," Browse...",NULL,this,ID_MINI_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,0,0);



   /* Label */
   new FXLabel(generalframe,"Name",NULL,JUSTIFY_LEFT);

   input = new FXTextField(generalframe,20,NULL,0,LAYOUT_SIDE_TOP|/*LAYOUT_CENTER_Y|*/LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
   FXLabel* fileType = new FXLabel(generalframe,"Type:",NULL,LAYOUT_SIDE_TOP|JUSTIFY_LEFT|LAYOUT_FILL_X);
   fileSize = new FXLabel(generalframe,"Size:",NULL,LAYOUT_SIDE_TOP|JUSTIFY_LEFT|LAYOUT_FILL_X);
   FXLabel *fileMod = new FXLabel(generalframe,"",NULL,LAYOUT_SIDE_TOP|JUSTIFY_LEFT|LAYOUT_FILL_X);

   strcpy(filename,(path+"/"+file).text());

   // Get file/link info
   if(lstat(filename,&info)!=0) return ;

   // Obtain user name
   fxgetusername(usrid,info.st_uid);
   // Obtain group name
   fxgetgroupname(grpid,info.st_gid);
   perm->user->setText(usrid);
   perm->grp->setText(grpid);
   oldgrp = grpid;
   oldusr = usrid;

   // If its a link, get the info on file itself
   islink=S_ISLNK(info.st_mode);
   if(islink && stat(filename,&info)!=0) return ;
   if(islink)
       extension="Link to ";
   else
       extension="";

   while(islink)
   {
       // Get file/link info
       if(lstat(filename,&info)!=0) return;

       // If its a link, get the info on file itself
       islink=S_ISLNK(info.st_mode);
       if(islink && stat(filename,&info)!=0) return;
       if(islink)
       {
           //      printf("Reading link...\n");
           int nread;
           nread=readlink(filename,linkto,sizeof(linkto)-1);
           if(nread>0)
           {
               linkto[nread]='\0';
               strcpy(filename,linkto);
           }
           else break;
       }
   }

   orig_mode=(info.st_mode & (S_IRWXU|S_IRWXG|S_IRWXO));

      // Mod time
   strncpy(mod,ctime(&info.st_mtime)+4,20);
   mod[20]=0;

      // Change time
   strncpy(changed,ctime(&info.st_ctime)+4,20);
   changed[20]=0;

      // Accesed time
   strncpy(accessed,ctime(&info.st_atime)+4,20);
   accessed[20]=0;

      //Size
   filesize=info.st_size;

   FXString fileassoc;
   pid=-1;
   fd[0]=-1;
   fd[1]=-1;
        // Is it a directory?
   if(S_ISDIR(info.st_mode))
   {
       char dupath[256];
       char buf[MAXPATHLEN+1];
       sprintf(dupath,"%s/%s",path.text(),file.text());
       if(pipe(fd))
           perror("can't create pipe:");

       extension+="Folder";

       switch(pid=fork())
       {
         case -1:
           perror("fork");
           // child process
         case 0:;
         if(fd[1]==-1) exit(1);
       
         off_t size=0;
         if(::du(dupath,size))
         {
           char buf[100];
           sprintf(buf,"%lu\n",size);
           write(fd[1],buf,strlen(buf)+1);
         }
         else write(fd[1],"0 bytes",8);
       

         if(close(fd[1])) exit(1);
         exit(0);
       }
   }
   else if(S_ISCHR(info.st_mode))
       extension+="Character Device";
   else if(S_ISBLK(info.st_mode))
       extension+="Block Device";
   else if(S_ISFIFO(info.st_mode))
       extension+="Named Pipe";
   else if(S_ISSOCK(info.st_mode))
       extension+="Socket";
   else // Regular file
   {
       // Try use association table
       fxfilepart(name,filename);
       fileassoc=getApp()->reg().readStringEntry("FILETYPES",name,"");
       e = name;
       if(fileassoc.empty())
       {
           e=strchr(name,'.');
           while(e)
           {
               fileassoc = getApp()->reg().readStringEntry("FILETYPES",e+1,"");
               if(!fileassoc.empty())
               {
                   e++;
                   break;
               }
               e=strchr(e+1,'.');
           }
       }

       if(!fileassoc.empty())// If we have an association
       {
           FXString c;
           extension+=fileassoc.extract(1,';');
           ext->setText(e);
           ext->disable();
           c=fileassoc.extract(0,';');
           descr->setText(fileassoc.extract(1,';'));
           open->setText(c.extract(0,','));
           view->setText(c.extract(1,','));
           if(edit)
               edit->setText(c.extract(2,','));
           bigic->setText(fileassoc.extract(2,';'));
           miniic->setText(fileassoc.extract(3,';'));

           fassoctab->enable();
       }
       else
       {
           if(info.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH))
               extension+="Application";
           else
           {
               extension+="Document";
               fassoctab->enable();
           }
       }
   }


   /* set file name, user name, group name */
   input->setText(file);
   input->setFocus();
   input->onCmdSelectAll(0,0,0);
  
   /* set permissions */

   perm->ur->setCheck((orig_mode & S_IRUSR) ? TRUE : FALSE);
   perm->uw->setCheck((info.st_mode & S_IWUSR) ? TRUE : FALSE);
   perm->ux->setCheck((info.st_mode & S_IXUSR) ? TRUE : FALSE);

   perm->gr->setCheck((info.st_mode & S_IRGRP) ? TRUE : FALSE);
   perm->gw->setCheck((info.st_mode & S_IWGRP) ? TRUE : FALSE);
   perm->gx->setCheck((info.st_mode & S_IXGRP) ? TRUE : FALSE);

   perm->cb_or->setCheck((info.st_mode & S_IROTH) ? TRUE : FALSE);
   perm->ow->setCheck((info.st_mode & S_IWOTH) ? TRUE : FALSE);
   perm->ox->setCheck((info.st_mode & S_IXOTH) ? TRUE : FALSE);
    
   perm->set->setCheck();
   /* set file info */

    strcpy(file_info,"Type:                    ");
    strcat(file_info,extension.text());
    fileType->setText(file_info);

    if(pid!=-1)
    {
            strcpy(file_info,"\nLast Modified:          ");
            strcat(file_info,mod);
            strcat(file_info,"\n\nLast Changed:         ");
            strcat(file_info,changed);
            strcat(file_info,"\n\nLast Accessed:        ");
            strcat(file_info,accessed);
            fileMod->setText(file_info);
            fileSize->setText("\nSize:                     calculating...");
    }
    else
    {
        strcpy(file_info,"\nSize:                     ");
        FXchar shortsize[50];
        sprintf(shortsize,"%ld",filesize);
        if(::Size(shortsize,shortsize))
            sprintf(file_info,"%s %s (%ld)",file_info,shortsize,filesize);
        else
            sprintf(file_info,"%s %s",file_info,shortsize);
        fileSize->setText(file_info);

        strcpy(file_info,"\nLast Modified:          ");
        strcat(file_info,mod);
        strcat(file_info,"\n\nLast Changed:         ");
        strcat(file_info,changed);
        strcat(file_info,"\n\nLast Accessed:        ");
        strcat(file_info,accessed);
        fileMod->setText(file_info);
    }
    
    mode = orig_mode;
    command = FXAttributesBox::ID_SET;//single file
    files = &file;
    orig_file = file;
    num = 1;
}

// Construct window for multiple files
FXAttributesBox::FXAttributesBox(FXApp *app,FXString *file,int n): FXDialogBox(app,"Attributes", DECOR_TITLE|DECOR_BORDER/*,0,0,350,200*/)
{ 
//    rec = 0;
    /* Buttons */
    FXHorizontalFrame *buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,0,0,10,10,5,5);

    /* Contents */
    FXVerticalFrame *contents=new FXVerticalFrame(this,LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);

    /* Cancel */
    new FXButton(buttons,"&Cancel",NULL,this,FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);

    /* Accept  */
    FXButton *ok = new FXButton(buttons,"&Accept",NULL,this,FXAttributesBox::ID_ACCEPT_MULT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
    ok->addHotKey(KEY_Return);
    /* Switcher */    
    FXTabBook *tabbook = new FXTabBook(contents,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);        


    /* Second item is Access Permissions */
    new FXTabItem(tabbook,"&Permissions",NULL);
    perm = new PermFrame(tabbook,this);
    perm->add->setCheck();

    mode = 0;
    command = ID_ADD;
    files = file;
    orig_file = "";
    num = n;
    pid=-1;
    fd[0]=-1;
    fd[1]=-1;

}
long FXAttributesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
{
    int rc=0,i;

    FXDialogBox::onCmdAccept(o,s,p);
    if(mode)
    {
        for(i=0;i<num;i++)
        {
            struct stat info;
            mode_t m;
            if(lstat(files[i].text(),&info)!=0)
                continue ;

            switch(command)
            {

            case FXAttributesBox::ID_ADD:
                m=info.st_mode|mode;
                break;
            case FXAttributesBox::ID_CLEAR:
                m=info.st_mode & ~mode;
                break;
            case FXAttributesBox::ID_SET:
                m=mode;
                break;
            default :
                return 1;//FXDialogBox::onCmdAccept(o,s,p);
            }
            rc|=chmod((char*)files[i].text(),m,perm->rec->getCheck(),perm->dironly->getCheck());
        }
        if(rc)
            FXMessageBox::error(this,MBOX_OK,"error", "Chmod failed: %s", strerror(errno));
    }
//   printf("old uid = %s\tnew uid = %s\n",oldusr.text(),perm->user->getText().text());
    //   printf("old gid = %s\tnew gid = %s\n",oldgrp.text(),perm->grp->getText().text());
    rc = 0;
   // chown if needed
   if(!perm->grp->getText().empty() || !perm->user->getText().empty())
   {
       for(i=0;i<num;i++)
       {
           struct stat info;
           mode_t m;
           if(lstat(files[i].text(),&info)!=0)
               continue ;

           uid_t uid = -1;
           gid_t gid = -1;
           struct passwd* pwde;
           while(pwde = getpwent())
               if(perm->user->getText() == pwde->pw_name)
                   uid = pwde->pw_uid;
           endpwent();

           struct group* grpe;
           while(grpe = getgrent())
               if(perm->grp->getText() == grpe->gr_name)
                   gid = grpe->gr_gid;
           endgrent();


           rc |= chown((char*)files[i].text(),uid,gid,perm->rec->getCheck(),perm->dironly->getCheck());
            if(rc)
            {
                FXMessageBox::error(this,MBOX_OK,"error", "Chown failed: %s", strerror(errno));
                return 1;
            }
       }
   }
    return 1;
}
long FXAttributesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
{
    int rc=0;
    if(pid!=-1)
    {
        if(kill(pid,9))
            perror("kill failed: ");
        unlink(tmp);
        pid = -1;
    }

    FXString oldfileassoc,fileassoc,op,v,e;

    op = open->getText();
    
    v = view->getText();
    if(!v.empty())
        v = "," + v;

    if(edit)
    {
        e = edit->getText();
        if(!e.empty())
            e = "," + e;
    }

    fileassoc = ext->getText();
    fileassoc += "=";
    fileassoc += op + v + e + ";";
    fileassoc += descr->getText() + ";";
    fileassoc += bigic->getText() + ";" + miniic->getText() + ";;";

    oldfileassoc=getApp()->reg().readStringEntry("FILETYPES",ext->getText().text(),"");
//   printf("old association: %s\n",oldfileassoc.text());
//   printf("new association: %s\n",fileassoc.extract(1,'=').text());
   if(oldfileassoc == "" || fileassoc.extract(1,'=') != oldfileassoc)
   {
      getApp()->reg().writeStringEntry("FILETYPES",ext->getText().text(),fileassoc.extract(1,'=').text());
//      printf("change association: %s\n:",fileassoc.extract(1,'=').text());
      getApp()->getMainWindow()->handle(this,MKUINT(FDAppMainWindow::ID_FILE_ASSOC,SEL_COMMAND),(void*)ext->getText().text());
   }


   /* rename file */
   
   if(orig_file != input->getText())
      if(rename(orig_file.text(),input->getText().text()))
         FXMessageBox::error(this,MBOX_OK,"error", "Rename failed: %s", strerror(errno));
   
   /* change perm */

   FXDialogBox::onCmdAccept(o,s,p);
   if(!(orig_mode == mode && command == FXAttributesBox::ID_SET))//no change
   {
//       return FXDialogBox::onCmdAccept(o,s,p);


       //FXDialogBox::onCmdAccept(o,s,p);

       struct stat info;
       mode_t m;
       if(lstat(input->getText().text(),&info)!=0)
           return 1;

       switch(command)
       {
       case FXAttributesBox::ID_ADD:
           m=info.st_mode|mode;
           break;
       case FXAttributesBox::ID_CLEAR:
           m=info.st_mode & ~mode;
           break;
       case FXAttributesBox::ID_SET:
           m=mode;
           break;
       default :
           return 1;
       }
       rc = chmod((char*)input->getText().text(),m,perm->rec->getCheck());
       if(rc)
           FXMessageBox::error(this,MBOX_OK,"error", "Chmod failed: %s", strerror(errno));
   }
//   printf("old uid = %s\tnew uid = %s\n",oldusr.text(),perm->user->getText().text());
//   printf("old gid = %s\tnew gid = %s\n",oldgrp.text(),perm->grp->getText().text());
   // chown if needed
   rc = 0;
   if(oldgrp != perm->grp->getText() || oldusr != perm->user->getText())
   {
       uid_t uid = -1;
       gid_t gid = -1;
       struct passwd* pwde;
       while(pwde = getpwent())
           if(perm->user->getText() == pwde->pw_name)
               uid = pwde->pw_uid;
       endpwent();

       struct group* grpe;
       while(grpe = getgrent())
           if(perm->grp->getText() == grpe->gr_name)
               gid = grpe->gr_gid;
       endgrent();



       rc = chown((char*)input->getText().text(),uid,gid,perm->rec->getCheck());
       if(rc)
           FXMessageBox::error(this,MBOX_OK,"error", "Chown failed: %s", strerror(errno));
   }
   return 1;
}

long FXAttributesBox::onCmdCommand(FXObject* o,FXSelector s,void* p)
{
/*
   FXCheckButton* ch = (FXCheckButton*)o;
   if(ch->getCheck())// check is on now
   {
       switch(SELID(s))
       {
       case FXAttributesBox::ID_ADD:
           fprintf(stderr,"FXAttributesBox::onCmdCommand ID_ADD\n");
           break;
       case FXAttributesBox::ID_CLEAR:
           fprintf(stderr,"FXAttributesBox::onCmdCommand ID_CLEAR\n");
           break;
       case FXAttributesBox::ID_SET:
           fprintf(stderr,"FXAttributesBox::onCmdCommand ID_SET\n");
           break;
       default :
           fprintf(stderr,"FXAttributesBox::onCmdCommand invalid command\n");
           break;
       }
   }
*/
   command = SELID(s);
   return 1;
}
long FXAttributesBox::onCmdCheck(FXObject* o,FXSelector s,void* p)
{
/*
    char *m;
    switch(SELID(s))
    {
    case S_IRUSR:
        m = "S_IRUSR";
        break;
    case S_IWUSR:
        m = "S_IWUSR";
        break;
    case S_IXUSR:
        m = "S_IXUSR";
        break;
    case S_IRGRP:
        m = "S_IRGRP";
        break;
    case S_IWGRP:
        m = "S_IWGRP";
        break;
    case S_IXGRP:
        m = "S_IXGRP";
        break;
    case S_IROTH:
        m = "S_IROTH";
        break;
    case S_IWOTH:
        m = "S_IWOTH";
        break;
    case S_IXOTH:
        m = "S_IXOTH";
        break;
    default:
        break;
    }
*/
    FXCheckButton* ch = (FXCheckButton*)o;
    if(!ch->getCheck())
    {
        mode = mode & ~SELID(s);
//        fprintf(stderr,"FXAttributesBox::onCmdCheck unset %s mode = %o\n",m,mode);
    }
    else
    {
        mode |= SELID(s);
//        fprintf(stderr,"FXAttributesBox::onCmdCheck set %s mode = %o\n",m,mode);
    }
    return 1;
}
long FXAttributesBox::onCmdBrowse(FXObject* o,FXSelector s,void* p)
{
    FXFileDialog browseProgram(this,"Select Program");
    if(browseProgram.execute())
    {
        char *path=(char*)browseProgram.getFilename().text();
        char file[MAXPATHLEN+1];
        //      if(!fxexists(path))
        //         return 0;
        fxfilepart(file,path);
        switch(SELID(s))
        {
        case ID_BROWSE_OPEN:
            open->setText(file);
            break;
        case ID_BROWSE_VIEW:
            view->setText(file);
            break;
        case ID_BROWSE_EDIT:
            if(edit)
                edit->setText(file);
            break;
        }
    }
    return 1;
}
long FXAttributesBox::onCmdBrowseIcon(FXObject* o,FXSelector s,void* p)
{
    FXString icon;
    if(SELID(s)==ID_BIG_ICON)
        icon = bigic->getText();
    else
        icon= miniic->getText();

    FXString iconpath=getApp()->reg().readStringEntry("SETTINGS","iconpath","/usr/local/lib/foxicons");
    const FXchar *patterns[]={
        "GIF Images",     "*.gif",
        "BMP Images",     "*.bmp",NULL};
        FXFileDialog browseIcon(this,"Select Icon");
        browseIcon.setFilename(iconpath+"/"+icon);
        browseIcon.setPatternList(patterns);
        if(browseIcon.execute())
        {
            char *path=(char*)browseIcon.getFilename().text();
            char file[MAXPATHLEN+1];
            if(!fxexists(path))
                return 0;
            fxfilepart(file,path);
            if(SELID(s)==ID_BIG_ICON)
                bigic->setText(file);
            else
                miniic->setText(file);
        }
        return 1;
}
long FXAttributesBox::onUpdWait(FXObject* o,FXSelector s,void* p)
{
    if(perm->rec->getCheck())
        perm->dironly->enable();
    else
        perm->dironly->disable();

    if(pid==-1)
        return 0;
    int status;
    if(waitpid(pid,&status,WNOHANG)/*==pid*/)
    {
        char buf[100];
        char size[100];
        unsigned long sz=0;
        int n=read(fd[0],size,sizeof(size));
        
        ::Size(size,size);
        sprintf(buf,"\nSize:                     %s",size);
        
        fileSize->setText(buf);
        close(fd[0]);
        pid=-1;
    }
}
long FXAttributesBox::onCmdCancel(FXObject* o,FXSelector s,void* p)
{
    if(pid!=-1)
    {
        if(kill(pid,9))
            perror("kill failed: ");
        unlink(tmp);
    }
   FXDialogBox::onCmdCancel(o,s,p);
   return 1;
}

int du(char *path, off_t &size)
{
    int rc=0;
    struct stat stats;
  if (lstat(path, &stats))//if it doesn't exist
  {
      perror("can't stat file:");
     return -1;
  }
//  size+=stats.st_size; // show all files
  /* Directory: stat recursively */
  if (S_ISDIR(stats.st_mode))
  {
    DIR *dir;
    struct dirent *entry;
    int i, pl = strlen(path);
  
    if (!(dir = opendir(path)))
    {
        perror("can't open dir:");
        return -1;
    }

    for(i = 0; (entry = readdir(dir)); i++)
      if (entry->d_name[0] != '.' || (entry->d_name[1] != '\0'
				      && (entry->d_name[1] != '.' ||
					  entry->d_name[2] != '\0'))) {
	int pl1 = pl, l = strlen(entry->d_name);
	char *path1 = (char *)alloca(pl1+l+2);

	strcpy(path1, path);
	if (path1[pl1-1] != '/')
	  path1[pl1++] = '/';
        strcpy(path1+pl1, entry->d_name);
	rc|=du(path1,size);
    }
    if (closedir(dir) || rc)
      return -1;
  }
  else // show only regular files' size
      size+=stats.st_size;

}












//------------------------------------------------------------------------------------








int FXAttributesBox::chmod(char *path,mode_t mode,FXbool rec,FXbool dironly)
{
    //hidden = TRUE;
    struct stat stats;
    if (lstat(path, &stats))//if it doesn't exist
    {
//        char str[100];
//        sprintf(str,"%s doesn't exist.", path);
//        showError(str);
        return -1;
    }

    if (!S_ISDIR(stats.st_mode))// file
    {
        if(dironly)
            return 0;

//        fprintf(stderr,"chmod file [%s]\n", path);
        return ::chmod(path,mode);
    }
    else// directory
    {
        if(rec == FALSE)
        {
//            fprintf(stderr,"chmod folder [%s]\n", path);
            if(::chmod(path,mode))// do not change recursively
            {
//                getApp()->showModalErrorBox(MBOX_OK,"error", "Chmod failed: %s", strerror(errno));
                return -1;
            }
        }
        else
            return rchmod(path,mode,dironly);// recurcive change
    }
    return 0;

}
int FXAttributesBox::rchmod(char *path,mode_t mode,FXbool dironly)
{
    struct stat stats;
    if (lstat(path, &stats))//if it doesn't exist
    {
//        char str[100];
//        sprintf(str,"%s doesn't exist.", path);
//        showError(str);
        return -1;
    }

    if (!S_ISDIR(stats.st_mode))// file
    {
        if(dironly)
            return 0;

//        fprintf(stderr,"chmod file [%s]\n", path);
        return ::chmod(path,mode);
    }

    // directory
//    fprintf(stderr,"chmod folder [%s]\n", path);
/*
    if(::chmod(path,S_IRUSR|S_IXUSR))
    {
//        getApp()->showModalErrorBox(MBOX_OK,"error", "Chmod failed: %s", strerror(errno));
        return -1;
    }
*/
    DIR *dir;
    struct dirent *entry;
    int i, pl = strlen(path);

    if (!(dir = opendir(path)))
        return -1;

    for(i = 0; (entry = readdir(dir)); i++)
        if (entry->d_name[0] != '.' || (entry->d_name[1] != '\0'
                                        && (entry->d_name[1] != '.' ||
                                            entry->d_name[2] != '\0'))) {
            int pl1 = pl, l = strlen(entry->d_name);
            char *path1 = (char *)alloca(pl1+l+2);

            strcpy(path1, path);
            if (path1[pl1-1] != '/')
                path1[pl1++] = '/';
            strcpy(path1+pl1, entry->d_name);
            if (rchmod(path1,mode,dironly))
            {
                /* take care of recursive errors */
//                char str[100];
//                sprintf(str,"Chmod error %s: %s", path, strerror(errno));
                //                showError(str);
                return -1;
            }
        }

    if (closedir(dir))
        return -1;

    return ::chmod(path,mode);
}
int FXAttributesBox::chown(char *path,uid_t uid,gid_t gid,FXbool rec,FXbool dironly)
{
    char str[100];

    //hidden = TRUE;
    struct stat stats;
    if (lstat(path, &stats))//if it doesn't exist
    {
//        char str[100];
//        fprintf(stderr,"%s doesn't exist.", path);
//        showError(str,FALSE);
        return -1;
    }
    
    if (!S_ISDIR(stats.st_mode))// file
    {
        if(dironly)
            return 0;

        if(::chown(path,uid,gid))
        {
//            fprintf(stderr,"Chmod error %s: %s", path, strerror(errno));
//            showError(str,FALSE);
            return -1;
        }
//        fprintf(stderr,"chowned file [%s]\n", path);
    }
    else// directory
    {
        if(rec == FALSE)
        {
            if(::chown(path,uid,gid))// do not change recursively
            {
//                sprintf(str,"Chmod error %s: %s", path, strerror(errno));
//                showError(str,FALSE);
                return -1;
            }
            fprintf(stderr,"chowned dir [%s]\n", path);
        }
        else
            if(rchown(path,uid,gid,dironly))// recurcive change
            {
//                fprintf(stderr,"Chmod error %s: %s", path, strerror(errno));
//                showError(str,FALSE);
                return -1;
            }
    }
    return 0;

}
int FXAttributesBox::rchown(char *path,uid_t uid,gid_t gid,FXbool dironly)
{
    char str[100];
    struct stat stats;
    if (lstat(path, &stats))//if it doesn't exist
    {
//        fprintf(stderr,"%s doesn't exist.", path);
//        showError(str,FALSE);
        return -1;
    }

    if (!S_ISDIR(stats.st_mode))// file
    {
        if(::chown(path,uid,gid))
        {
            if(dironly)
                return 0;
//            fprintf(stderr,"Chmod error %s: %s", path, strerror(errno));
//            showError(str,FALSE);
            return -1;
        }
        return 0;
    }

    // directory
//    if(::chown(path,uid,gid))
//    {
//        sprintf(str,"Chmod error %s: %s", path, strerror(errno));
//        showError(str,FALSE);
//        return -1;
//    }

    DIR *dir;
    struct dirent *entry;
    int i, pl = strlen(path);

    if (!(dir = opendir(path)))
    {
//        fprintf(stderr,"can't open directory %s: %s", path, strerror(errno));
        return -1;
    }

    for(i = 0; (entry = readdir(dir)); i++)
        if (entry->d_name[0] != '.' || (entry->d_name[1] != '\0'
                                        && (entry->d_name[1] != '.' ||
                                            entry->d_name[2] != '\0'))) {
            int pl1 = pl, l = strlen(entry->d_name);
            char *path1 = (char *)alloca(pl1+l+2);

            strcpy(path1, path);
            if (path1[pl1-1] != '/')
                path1[pl1++] = '/';
            strcpy(path1+pl1, entry->d_name);
            if (rchown(path1,uid,gid,dironly))
            {
                /* take care of recursive errors */
//               fprintf(stderr,"Chmod error %s: %s", path, strerror(errno));
//                showError(str,FALSE);
                return -1;
            }
        }

    if (closedir(dir))
    {
//        fprintf(stderr,"can't close directory %s: %s", path, strerror(errno));
        return -1;
    }

    if(::chown(path,uid,gid))
    {
//        fprintf(stderr,"Chmod error %s: %s", path, strerror(errno));
        return -1;
    }
    return 0;
}
