/* foXcommander - foXdesktop file manager
 * Copyright (C) 2000-2001 Josua Groeger derived from XWC (C) 1999-2000 Maxim Baranov
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef FDAPPMAINWINDOW_H
#define FDAPPMAINWINDOW_H

#ifndef PATHMAX
#define PATHMAX 1024
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "Panel.h"
#include "Attributes.h"
#include <foXdesktop/foXdesktop.h>

/* Application object */
class FDAppMainWindow : public FXMainWindow {
    FXDECLARE(FDAppMainWindow)
    friend class Panel;
public:
  long onQuit(FXObject*,FXSelector,void*);
  long onCmdAbout(FXObject*,FXSelector,void*);
  long onCmdHelp(FXObject*,FXSelector,void*);
  long onCmdFileAssoc(FXObject*,FXSelector,void*);  
  long onCmdMenu(FXObject*,FXSelector,void*);  
  long onCmdOptions(FXObject*,FXSelector,void*);  
  long onCmdDirBox(FXObject*,FXSelector,void*);
  long onUpdDirBox(FXObject*,FXSelector,void*);
  long onCmdRun(FXObject*,FXSelector,void*);
  long onCmdXTerm(FXObject*,FXSelector,void*);  
  long onCmdSelect(FXObject*,FXSelector,void*);
  long onCmdDirectory(FXObject*,FXSelector,void*);
  long onCmdFileCopy(FXObject*,FXSelector,void*);
  long onCmdShow(FXObject*,FXSelector,void*);
  long onUpdShow(FXObject*,FXSelector,void*);
  long onCmdBookmark(FXObject*,FXSelector,void*);

  long onCmdOptionsChanged(FXObject*,FXSelector,void*);
  long onCmdHideTree(FXObject*,FXSelector,void*);
  long onCmdToggleHidden(FXObject*,FXSelector,void*);
  

protected:
    FDAppMainWindow () {};
    int RunHistSize;
    FXbool toolbar_style;
#define RUN_HIST_SIZE 15
    char RunHistory[RUN_HIST_SIZE][100];

    FXIcon *archicon;
    FXIcon *attricon;

    FXLabel  *lab;
//    FXMainWindow*      mainwindow;
    FXVerticalFrame   *group1;
    FXMenubar*         menubar;
    FXToolbar* toolbar;
    FXStatusbar*       status;
    FXDirBox*          dirbox;
    FXDirList*         dirlist;
    Panel*             lpanel;
    Panel*             rpanel;
    FXString           target;
    FXString           sources;
    char               *opendir;
    FXMenuPane         *leftviewmenu;
    FXMenuPane         *rightviewmenu;
    FXMenuTitle        *rmenutitle;
    FXMenuTitle        *lmenutitle;
    FXRecentFiles      *bookmarks;

    FXToggleButton *hideButton;

    FXDialogBox *preferencesDialog;
    FDComponentConstructDict *constructDict;

public:
  enum{
    ID_ABOUT=FXMainWindow::ID_LAST,
    ID_HELP,
    ID_XTERM,
    ID_DIRECTORYLIST,
    ID_SELECT_ALL,
    ID_DESELECT_ALL,
    ID_SELECT_INVERSE,
    ID_FILE_COPY,
    ID_FILE_MOVE,
    ID_FILE_LINK,
    ID_FILE_SLINK,
    ID_FILE_DEL,
    ID_RUN,
    ID_OPTIONS,
    ID_AUTO_SAVE,
    ID_FILE_ASSOC,
    ID_DIR_BOX,
    ID_MENU,
    ID_SHOW_BIG_ICONS,
    ID_SHOW_MINI_ICONS,
    ID_SHOW_DETAILS,
    ID_BOOKMARK,
    ID_ADD_BOOKMARK,
    ID_QUIT,
    ID_OPTIONS_CHANGED,
    ID_HIDE_TREE,
    ID_TOGGLE_HIDDEN,
    ID_LAST
    };
public:
  FDAppMainWindow (FDApp *a, const FXString &name, FXIcon *ic= NULL, FXIcon *mi= NULL,
           FXuint opts= DECOR_ALL, FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  virtual ~FDAppMainWindow ();
  void create();
  void saveConfig();
  FXFileDict *getAssociations(){return lpanel->current->list->getAssociations();}
};
#endif
